#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 01.08.2013

import threading, time
from global_h import *
from ser_port import open_ser_port

class ATMega_i(threading.Thread):
    """ Empfängt im Hintergrund Daten via Serielle Schnittstelle

        Methoden:
        twiread(twiadr, cnt = 1, regadr = None)
        twiwrite(twiadr, data = [], regadr = None)
        add_callback(callback, frame = None, twiadr = None)
        - fügt ein Callback für ein Frame oder eine TwiAdr ein
        del_callback(frame = None, twiadr = None)
        - entfernt ein Callback für ein Frame oder eine TwiAdr.

        Lauscht im Hintergrund an der Seriellen Schnittstelle
        und reicht empfangene und verifizierte, gültige Daten
        via verschiedener Callbacks an das Hauptprogramm weiter.

        Die Daten beginnen mit dem immer gleichen 3-Byte Header:
        STARTBYTE, FRAMETYP, FRAMELÄNGE
        der Rest ist abhängig vom Frametyp:
        TWOWIRE   -> TWI-Adresse, Datenbyte.0 ......
        ONEWIRE   -> Pinmaske, Temp_high, Temp_low, ... , ...
        PORTREAD  -> PORTNR, DDR-Status, PORT-Status, PIN-Status
        ADCREAD   -> Pin_Nr, ADC_high, ADC_low
        INTMODE   -> INT_Nr, Pin_Status
        DATAMODE  -> Datenbyte.0, ....

        Fehlermeldungen: liefern fehlerverursachende Parameter zurück
        TWI_ERR   -> TWI-Adresse
        OWI_ERR   -> Pinmaske
        ADC_ERR   -> gewählte Pin_Nr
        INT_ERR   -> gewählte Pin_Nr

        Es gibt zwei Dictionarys für die Callback-Funktionen:
        self.cb enthält Einträge für die verschiedenen Frames.

        Ein Sonderfall ist der Frame TWOWIRE vorgesehen:
        ist für TWOWIRE kein Eintrag in self.cb vorhanden,
        dann wird im Dictionray self.twi_cb gesucht.
        Hier kann für jede TWI-Adresse eine eigene Callback-Funktion
        vereinbart werden.

        Ist aber eine Zuordnung von TWOWIRE in self.cb definiert, dann
        wird diese für jede TWI-Adresse ausgeführt und self.twi_cb wird ignoriert.
        """

    def __init__(self, ser):
        threading.Thread.__init__(self)
        self.s = ser
        self.__ende = False
        self.err = False
        self.is_reading = False
        self.cb = {}        # ein leeres Dictionary für Frames
        self.twi_cb = {}    # ein Dictionary für TWI-Adressen

        self.start()

    def add_callback(self, cb, frame = None, twiadr = None):
        """ Fügt eine Callback-Funktion in das Dictonary cb / twi_cb ein

            Es darf nur frame oder twiadr übergeben werden
            Ist der Schlüssel bereits vorhanden,
            dann wird der Wert überschrieben """
        if twiadr is None and frame is None:
            raise SyntaxError("Frame oder TWI-Adresse angeben")

        if twiadr is not None and frame is not None:
            raise SyntaxError("Entweder Frame ODER TWI-Adresse angeben")

        if twiadr is None:
            i = self.cb.get(frame, None)
            if i is None:
                self.cb.setdefault(frame, cb)
            else:
                self.cb[frame] = cb
        else:
            i = self.twi_cb.get(twiadr, None)
            if i is None:
                self.twi_cb.setdefault(twiadr, cb)
            else:
                self.twi_cb[twi_adr]  = cb


    def del_callback(self, frame = None, twiadr = None):
        """ Entfernt eine Callback-Funktion aus den Dictionarys der Callbacks

            es darf nur frame oder twiadr übergben werden """

        if twiadr is None and frame is None:
            raise SyntaxError("Frame oder TWI-Adresse angeben")

        if twiadr is not None and frame is not None:
            raise SyntaxError("Entweder Frame ODER TWI-Adresse angeben")

        if twiadr is None:
            try:
                self.cb.pop(frame)
            except:
                print("key {} nicht in cb vorhanden".format(frame))
        else:
            try:
                self.twi_cb.pop(twiadr)
            except:
                print("key {} nicht in twi_cb vorhanden".format(twiadr))


    def run(self):
        self.s.flushInput()

        while self.__ende == False:
            self.busy = self.s.inWaiting()
            # Mindestens 3 Byte müssen gelesen werden
            # um den Header zu idenifizieren und die nachfolgende
            # Datenlänge zu ermitteln.
            # Zuerst wird nur der Identifier geprüft
            if self.busy > 0:
                self.err = True
                v = self.s.read(1)

                # das erste Byte muss 0xA5 sein
                # wenn nicht, dann liegt kein Datensatzanfang vor
                if not v[0] == STARTBYTE:
                    print("Falsches Startbyte: {:4x}".format(v[0]))
                    continue

                while self.s.inWaiting() < 2:
                    time.sleep(0.001)       # 1ms warten

                v = self.s.read(2)        # die nächsten 2 Byte des Headers einlesen
                data = self.s.read(v[1])  # danach die Daten einlesen
                if len(data) is 0:          # beenden, falls nichts empfangen wird
                    print("Keine Daten für Frame {}, Länge {} empfangen".format(v[0], v[1]))
                    continue

                if (v[0] is TWOWIRE) or (v[0] is PORTREAD) or (v[0] is ADCREAD):
                    self.is_reading = False

                # prüfen, ob für den Frametyp ein Eintrag
                # im Dict self.cb vorhanden ist

                cb = self.cb.get(v[0], None)
             #   print(self.cb)
                if cb is not None:
                    self.err = False

                    #print(self.cb.keys())
                    cb(data)                # die Daten weiterreichen
                else:
                # wenn kein Eintrag gefunden, dann
                # bei einem TWI-Transfer prüfen,
                # ob ein Eintrag in self.twi_cb vorhanden ist
                    if v[0] is TWOWIRE:
                        self.is_reading = False
                        #print(self.twi_cb)
                        cb = self.twi_cb.get((data[0] & 0xFE), None)
                        #print(cb)
                        if cb is not None:
                            self.err = False
                            cb(data)        # die Daten weiterreichen
                        else:
                            print("TWI-Adr 0x{:2x} ohne Callback".format(data[0]))

                    elif v[0] == TWI_ERR:
                        self.is_reading = False
                        print("TWI-Fehler @ Adr: 0x{:02x}".format(data[0]))
                    elif v[0] == OWI_ERR:
                        print("Ungültige OWI-Maske: 0b{:02b}".format(data[0]))
                    elif v[0] == ADC_ERR:
                        print("Ungültiger ADC-Pin: {}".format(data[0]))
                    elif v[0] == INT_ERR:
                        print("Ungültiger INT-Pin: {}".format(data[0]))
                    else:
                        print("Kein Callback für Frame: 0x{:02x}, empfangen: {}".format(v[0], data))

            time.sleep(0.005)

        if not self.s.closed:
            self.s.close()
        print("Thread ATMega stopped")
        exit()

    def twiwrite(self, twiadr,  data = [], regadr = None):
        """ Schreibt Daten auf ein TWI-Device

            Parameter 1: TWI-Adresse
            Parameter 2: optional Registeradresse
            Parameter 3: Daten als Iterable """
        out = [STARTBYTE, TWOWIRE, 0x01, twiadr]

        if regadr is not None:
            if type(regadr) is int:
                regadr = (regadr,)
            for i in regadr:
                out.append(i)
            out[2] += len(regadr)

        if type(data) is int:
            data = (data,)

        for i in data:
            out.append(i)
        out[2] += len(data)
        self.s.write(out)

    def twiread(self, twiadr, cnt = 1, regadr = None):
        """ Holt Daten von einem TWI-Device ab

            Parameter 1: TWI-Adresse
            Parameter 2: Anzahl der Bytes, die vom Slave ausgelesen werden sollen
            Parameter 3: optional: Registeradressen, die vorher an den Slave geschickt werden
                         Übergabe als Byte oder Iterable
            Rückgabe   : keine """

        self.is_reading = True

        out = [STARTBYTE, TWOWIRE, 0x02, (twiadr | 0x01)]
        if regadr is not None:
            if type(regadr) is int:
                regadr = (regadr,)
            for i in regadr:
                out.append(i)
            out[2] += len(regadr)

        out.append(cnt)
        self.s.write(out)


    def setint(self, irq, sense):
        """ setzt einen Interrupt am ATMEGA """
        self.s.write((STARTBYTE, INTMODE, 0x02, irq, sense))

    def ds18b20read(self, pinmask):
        """ Stößt eine Temperaturmessung mit DS18B20 an, die direkt an PortD
            des Atmega angeschlossen sind """
        self.s.write( (STARTBYTE, ONEWIRE, 0x01, pinmask) )

    def adcread(self, pin):
        """ Führt eine Messung an einem einzelnen ADC-Pin aus """
        self.is_reading = True
        self.s.write( (STARTBYTE, ADCREAD, 0x01, pin) )

# --- Ports komplett schalten --------------------------------------------------

    def port_mode(self, port, pinmask, pinstates):
        """ Einen oder mehrere Pins eines Ports als Ausgang oder Eingang setzen

            port (PORTB, PORTC, PORTD): 1, 2, 3
            pinmask   - Pins  des Ports, die geändert werden sollen
            pinstates - Status der Pins (im DDR-Register),
                    die in pinmask als zu ändernd angegeben sind

            pin_mode(0x01, 0x0F, 0x03) setzt an PORTB PB.0/PB1 als Ausgang und
                        PB.2/PB.3 als Eingang
            Rückgabe: keine """

        if not port in (1,2,3):
            raise SyntaxError("ungültiger Port")

        self.s.write( (STARTBYTE, PORTMODE, 0x03, port, pinmask, pinstates) )


    def port_read(self, port):
        """ Liest den Status der aller Pins des Ports

            port (PORTB, PORTC, PORTD): 1, 2, 3
            port_read(1) liest PortB

            Rückgabe: STARTBYTE, PORTREAD, 0x04, Port, DDR-Register, PORT-Register, PIN-Register
            """

        if not port in (1,2,3):
            raise SyntaxError("ungültiger Port")

        self.is_reading = True
        self.s.write( (STARTBYTE, PORTREAD, 0x01, port) )


    def port_write(self, port, pinmask, pinvalues):
        """ Setzt oder löscht einen oder mehrere Pins eines Ports

            port (PORTB, PORTC, PORTD): 1, 2, 3
            pinmask   - Pins  des Ports, die geändert werden sollen
            pinvalues - Werte der Pins, die in pinmask als zu ändernd angegeben sind

            pin_write(0x01, 0x0F, 0x3) setzt an PORTB PB.0/PB1 und löscht PB.2/PB.3

            Rückgabe: keine """

        if not port in (1,2,3):
            raise SyntaxError("ungültiger Port")

        self.s.write( (STARTBYTE, PORTWRITE, 0x03, port, pinmask, pinvalues) )

    def port_toggle(self, port, pinmask):
        """ Einen oder mehrere Pins eines Ports umschalten

            port (PORTB, PORTC, PORTD): 1, 2, 3
            pinmask   - Pins  des Ports, die geändert werden sollen

            pin_toggle(0x01, 0x04) wechselt den Status von PB.3

            Rückgabe: keine """

        if not port in (1,2,3):
            raise SyntaxError("ungültiger Port")
        self.s.write( (STARTBYTE, PORTTOG, 0x02, port, pinmask) )

# --- Pins schalten durch Angabe von Port und Pin -----------------------------

    def pin_mode(self, port, pin_nr, state):
        """ Schaltet den IO-Modus von genau einem Pin eines Ports um

            Paramter: Port, Pin_Nr, state [0 | <>0]
            Rückgabe: keine
            Fehlermeldung, wenn Port oder Pin ungültig """

        self.check_port_pin(port, pin_nr)
        pin_nr = (1<<pin_nr)
        if state:
            state = pin_nr

        self.s.write((STARTBYTE, PORTMODE, 0x03, port, pin_nr, state))


    def pin_read(self, port, pin_nr):
        """ Liest den Status von genau einem Pin eines Ports

            Rückgabe Port, Pin, 0/1
            port (PORTB, PORTC, PORTD): 1, 2, 3
            pin_nr[0...7]

            Rückgabe: keine
            Fehlermeldung, wenn Port oder Pin ungültig """

        self.check_port_pin(port, pin_nr)
        self.is_reading = True
        self.s.write( (STARTBYTE, PORTREAD, 0x01, port) )


    def pin_write(self, port, pin_nr, value):
        """ Schreibt genau einen Pin eines Ports

            Parameter: port, pin_nr, wert [0 | <>0]
            Rückgabe: keine
            Fehlermeldung, wenn Port oder Pin ungültig """

        self.check_port_pin(port, pin_nr)

        pin_nr = (1<<pin_nr)
        if value:
            value = pin_nr

        self.s.write((STARTBYTE, PORTWRITE, 0x03, port, pin_nr, value))

    def pin_tog(self, port, pin_nr):
        """ Schaltet den Status um von genau einem Pin eines Ports

            Paramter: Port, Pin_Nr
            Rückgabe: keine
            Fehlermeldung, wenn Port oder Pin ungültig """

        self.check_port_pin(port, pin_nr)

        self.s.write( (STARTBYTE, PORTTOG, 0x02, port, (1<<pin_nr)) )


    def check_port_pin(self, port, pin_nr):
        """ Pruft, ob die Angaben zu Port und Pin korrekt sind """

        if port is 1:
            if not (pin_nr >= 0 and pin_nr < 6):
                raise SyntaxError("PortB, ungültiger Pin: {}".format(pin_nr) )

        elif port is 2:
            if not (pin_nr >= 0 and pin_nr < 4):
                raise SyntaxError("PortC, ungültiger Pin: {}".format(pin_nr) )

        elif port is 3:
            if not (pin_nr >= 2 and pin_nr < 8):
                raise SyntaxError("PortD, ungültiger Pin: {}".format(pin_nr) )
        else:
            raise SyntaxError("unbekannter Port # {}".format(port))

    def stop(self):
        """ Beendet den Thread """
        self.__ende = True
