#!/usr/bin/python3
# -*- coding: UTF-8 -*-

import time
from global_h import *

class ATMega_p(object):
    """ Eine Klasse, über die auf alle Funktionen des ATMEGA88 zugegriffen werden kann.

        Methoden:
        - twiwrite()
        - twiread()
        - adcread()
        - ds18b20read()
        Einstellungen für Ports
        - port_mode()
        - port_read()
        - port_write()
        - port_tog()
        Einstellungen für Pins
        - pin_mode()
        - pin_read()
        - pin_write()
        - pin_tog()

        Nach dem Senden eines Kommandes (auf das eine Antwort erwartet wird)
        wird pollend die Serielle Schnittstelle abgefragt.
        Geht innerhalb der Wartezeit keine Antwort (oder eine Fehlermeldung ein),
        dann wird als Ergebnis None zurückgeliefert und ein Fehlerflag gesetzt.
        """
    def __init__(self, ser = None):
        self.s = ser
        self.err = False

    def twiwrite(self, twiadr, data = [], regadr = None ):
        """ Schreibt Daten auf ein TWI-Device

            Parameter 1: TWI-Adresse
            Paremeter 2: Daten als Int / Iterable
            Parameter 3: optionale Registeradresse  als Int / Iterable
        """
        out = [STARTBYTE, TWOWIRE, 0x01, (twiadr & 0xFE)]
        # Register-Adresse übernehmen
        if regadr is not None:
            if type(regadr) is int:
                regadr = (regadr,)
            for i in regadr:
                out.append(i)
            out[2] += len(regadr)

        # Daten übernehmen
        if type(data) is int:
            data = (data,)
        for i in data:
            out.append(i)
        out[2] += len(data)
        self.s.write(out)

    def twiread(self, twiadr, cnt = 1, regadr = None):
        """ Holt Daten von einem TWI-Device ab

            Parameter 1: TWI-Adresse
            Parameter 2: Anzahl der Bytes, die vom Slave ausgelesen werden sollen
            Parameter 3: optional: Registeradressen, die vorher an den Slave geschickt werden
                         Übergabe als Byte oder Iterable
             Rückgabe   : Bytefolge """
        out = [STARTBYTE, TWOWIRE, 0x02, (twiadr | 0x01)]
        if regadr is not None:
            if type(regadr) is int:
                regadr = (regadr,)
            for i in regadr:
                out.append(i)
            out[2] += len(regadr)

        out.append(cnt)
        self.s.write(out)

        return self.__poll()

    def adcread(self, pin):
        """ Liest einen ADC-Pin an PortC aus """
        self.s.write((STARTBYTE, ADCREAD, 0x01, pin))
        return self.__poll()

    def ds18b20read(self, mask):
        self.s.write((STARTBYTE, ONEWIRE, 0x01, mask))
        return self.__poll()

# --- Ports komplett schalten --------------------------------------------------

    def port_mode(self, port, pinmask, pinstates):
        """ Einen oder mehrere Pins eines Ports als Ausgang oder Eingang setzen

            port (PORTB, PORTC, PORTD): 1, 2, 3
            pinmask   - Pins  des Ports, die geändert werden sollen
            pinstates - Status der Pins (im DDR-Register),
                    die in pinmask als zu ändernd angegeben sind

            pin_mode(0x01, 0x0F, 0x03) setzt an PORTB PB.0/PB1 als Ausgang und
                        PB.2/PB.3 als Eingang
            Rückgabe: keine """

        if not port in (1,2,3):
            raise SyntaxError("ungültiger Port")

        self.s.write( (STARTBYTE, PORTMODE, 0x03, port, pinmask, pinstates) )


    def port_read(self, port):
        """ Liest den Status der aller Pins des Ports

            port (PORTB, PORTC, PORTD): 1, 2, 3
            port_read(1) liest PortB

            Rückgabe: STARTBYTE, PORTREAD, 0x04, Port, DDR-Register, PORT-Register, PIN-Register
            """

        if not port in (1,2,3):
            raise SyntaxError("ungültiger Port")

        self.s.write( (STARTBYTE, PORTREAD, 0x01, port) )
        return self.__poll()


    def port_write(self, port, pinmask, pinvalues):
        """ Setzt oder löscht einen oder mehrere Pins eines Ports

            port (PORTB, PORTC, PORTD): 1, 2, 3
            pinmask   - Pins  des Ports, die geändert werden sollen
            pinvalues - Werte der Pins, die in pinmask als zu ändernd angegeben sind

            pin_write(0x01, 0x0F, 0x3) setzt an PORTB PB.0/PB1 und löscht PB.2/PB.3

            Rückgabe: keine """

        if not port in (1,2,3):
            raise SyntaxError("ungültiger Port")

        self.s.write( (STARTBYTE, PORTWRITE, 0x03, port, pinmask, pinvalues) )

    def port_toggle(self, port, pinmask):
        """ Einen oder mehrere Pins eines Ports umschalten

            port (PORTB, PORTC, PORTD): 1, 2, 3
            pinmask   - Pins  des Ports, die geändert werden sollen

            pin_toggle(0x01, 0x04) wechselt den Status von PB.3

            Rückgabe: keine """

        if not port in (1,2,3):
            raise SyntaxError("ungültiger Port")
        self.s.write( (STARTBYTE, PORTTOG, 0x02, port, pinmask) )

# --- Pins schalten durch Angabe von Port und Pin -----------------------------

    def pin_mode(self, port, pin_nr, state):
        """ Schaltet den IO-Modus von genau einem Pin eines Ports um

            Paramter: Port, Pin_Nr, state [0 | <>0]
            Rückgabe: keine
            Fehlermeldung, wenn Port oder Pin ungültig """

        self.check_port_pin(port, pin_nr)
        pin_nr = (1<<pin_nr)
        if state:
            state = pin_nr

        self.s.write((STARTBYTE, PORTMODE, 0x03, port, pin_nr, state))


    def pin_read(self, port, pin_nr):
        """ Liest den Status von genau einem Pin eines Ports

            Rückgabe Port, Pin, 0/1
            port (PORTB, PORTC, PORTD): 1, 2, 3
            pin_nr[0...7]

            Rückgabe: 0/1
            Fehlermeldung, wenn Port oder Pin ungültig """

        self.check_port_pin(port, pin_nr)

        self.s.write( (STARTBYTE, PORTREAD, 0x01, port) )
        a = self.__poll()
        # Pin maskieren, liefert 0 oder 1 zurück
        a = (a[2] & (1<<pin_nr))
        if a > 0:
            a = 1
        return a

    def pin_write(self, port, pin_nr, value):
        """ Schreibt genau einen Pin eines Ports

            Parameter: port, pin_nr, wert [0 | <>0]
            Rückgabe: keine
            Fehlermeldung, wenn Port oder Pin ungültig """

        self.check_port_pin(port, pin_nr)

        pin_nr = (1<<pin_nr)
        if value:
            value = pin_nr

        self.s.write((STARTBYTE, PORTWRITE, 0x03, port, pin_nr, value))

    def pin_tog(self, port, pin_nr):
        """ Schaltet den Status um von genau einem Pin eines Ports

            Paramter: Port, Pin_Nr
            Rückgabe: keine
            Fehlermeldung, wenn Port oder Pin ungültig """

        self.check_port_pin(port, pin_nr)

        self.s.write( (STARTBYTE, PORTTOG, 0x02, port, (1<<pin_nr)) )


    def check_port_pin(self, port, pin_nr):
        """ Pruft, ob die Angaben zu Port und Pin korrekt sind """

        if port is 1:
            if not (pin_nr >= 0 and pin_nr < 6):
                raise SyntaxError("PortB, ungültiger Pin: {}".format(pin_nr) )

        elif port is 2:
            if not (pin_nr >= 0 and pin_nr < 4):
                raise SyntaxError("PortC, ungültiger Pin: {}".format(pin_nr) )

        elif port is 3:
            if not (pin_nr >= 2 and pin_nr < 8):
                raise SyntaxError("PortD, ungültiger Pin: {}".format(pin_nr) )
        else:
            raise SyntaxError("unbekannter Port # {}".format(port))


    def __poll(self):
        """ Pollt die Serielle Schnittstelle für einige Millisekunden.
            Gehen bis dahin keine Daten ein, wird ein IOError geworfen.
            liefert immer eine Bytefolge zurück """

        self.s.flushInput()
        cnt = 500
        self.err = False
        while cnt > 0:
            i = self.s.inWaiting()
            # Mindestens 3 Byte müssen gelesen werden
            # um den Header zu idenifizieren und die nachfolgende
            # Datenlänge zu ermitteln.
            if i > 0:
                v = self.s.read(1)
                #print(v)
                # das erste Byte muss 0xA5 sein
                # wenn nicht, dann liegt kein gültiger Datensatz von
                if not v[0] == STARTBYTE:
                    continue
                # warten bis der Header komplett ist
                while self.s.inWaiting() < 2:
                    time.sleep(0.001)

                # den restliche Header einlesen
                v = self.s.read(2)
                # und die Daten einlesen
                data = self.s.read(v[1])

                if v[0] == TWI_ERR:
                    self.err = True
                    print("TWI-Fehler @ Adr: 0x{:02x}".format(data[0]))
                    return None

                elif v[0] == OWI_ERR:
                    print("Ungültige OWI-Maske: 0b{:02b}".format(data[0]))
                    self.err = True
                    return None

                elif v[0] == ADC_ERR:
                    print("Ungültiger ADC-Pin: {}".format(data[0]))
                    self.err = True
                    return None

##                elif v[0] == INT_ERR:
##                    print("Ungültiger INT-Pin: {}".format(data[0]))
##                    self.err = True
##                    return None

                if v[0] == TWOWIRE:
                    #print("data: ", data)
                    # die TWI-Adresse nicht zurückliefern
                    return data[1:]

                else:
                    return data

            time.sleep(0.005)
            cnt -= 1

        if cnt is 0:
            self.err = True
            print("Fehler: keine Antwort")
            return None
