#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

# Humidity- und Temperatursensor HYT221

# hier: Auslesen des Sensors via Interrupt

# Es ist keine Adressierung von internen Registern erforderlich.
#
# Die Messung wird angestoßen durch die Adressierung des Sensors ohne Daten.
# Nach der Konversionzeit von 100ms können 4 Byte gelesen und interpretiert werden.
#
# Um den Abschluss der Konversion zu prüfen, werden im Byte[0] die oberen
# beiden Bits getestet, es darf keines der beiden Bits gesetzt sein.

import tkinter as Tk
import threading, time
from ser_port import open_ser_port
from global_h import *
from atmega_i import ATMega_i

class HYT221(threading.Thread):
    """ Hygrosens HYT221 Sensor für Luftfeuchtigkeit und Temperatur
        läuft als Hintergrundthread und stößt in regelmäßigen
        Zeitabständen Messungen an.
        Die Messwerte werden vom Thread Receiver empfangen und
        via Callback zur Anzeige gebracht """

    def __init__(self, twi_adr = 80, cb = None, sleeptime = 1):
        threading.Thread.__init__(self)
        self.twi_adr = twi_adr
        self.cb = cb
        self.ende = False
        self.sleeptime = sleeptime

    def run(self):
        while self.ende == False:
            # Messung anstoßen, dazu nur die Adresse senden
            uc.twiwrite(self.twi_adr)
            # Konversionszeit abwarten
            time.sleep(0.1)
            # Das Auslesen veranlassen, den Rest macht run()
            uc.twiread(self.twi_adr, cnt = 4)
            # Wartezeit zwischen den Messungen
            time.sleep(self.sleeptime)

        print("Thread HYT221 stopped")
        exit()

    def calc(self, data):
        """ Berechnet die Temperatur aus dem gelieferten Messwert
            bei gesetztem höchstwertigen Bit liegen negative Temperaturen vor
            je Digit : 0.0625 Grad  """
        if data[0] & 0b11000000:
            print("hyt-error")
        humi = round(((data[0] * 256 + data[1]) * 100 / 16384), 1)
        temp = round(((data[2] * 256 + data[3]) * 165 / 65536 - 40 ),1)

        return (humi, temp)

    def stop(self):
        """ Stopt den Thread """
        self.ende = True

    def set_sleeptime(self, time):
        """ stellt die Wartezeit zwischen den Messungen ein """
        self.sleeptime = time


class Hygrometer(object):
    """ Ausgabewindow für die Messwerte """
    def __init__(self, master):
        self.tvar = Tk.StringVar()
        self.hvar = Tk.StringVar()
        self.tvar.set("--")
        self.hvar.set("--")

        Tk.Label(master,text = "Humi", width = 5, anchor = "e", font = ('Arial', 20)).grid(
                column = 0, row = 0, padx = 5, pady = 5, sticky = "nse")
        Tk.Label(master,text = "Temp", width = 5, anchor = "e", font = ('Arial', 20)).grid(
                column = 0, row = 1, padx = 5, pady = 5, sticky = "nse")
        self.t = Tk.Label(master, width = 8, textvariable = self.hvar, font = ('Arial',20))
        self.t.grid(column = 1, row = 0, sticky = "nsw")
        self.t = Tk.Label(master, width = 8, textvariable = self.tvar, font = ('Arial',20))
        self.t.grid(column = 1, row = 1, sticky = "nsw")

    def set(self, value):
        self.hvar.set("{:3} %".format(value[0]))
        self.tvar.set("{:4.1f} C°".format(value[1]))


if __name__ == "__main__":

    def shutdown(args = None):
        if uc.is_alive():
            uc.stop()
        if hyt.is_alive():
            hyt.stop()

    def get_hyt(data):
        """ Holt die eingegangenen TWI-Daten ab, rechnet die Messwerte um
            und sendet das Ergebnis an die Anzeige """
        #print(data)
        # die TWI-Adresse des Absenders als Byte.0 wird entfernt
        val = hyt.calc(data[1:])
        h.set(val)

    top = Tk.Tk()
    top.title("HYT221")
    top.bind("<Destroy>", shutdown)

    frm = Tk.LabelFrame(top, text = "Humidity & Temperature")
    frm.grid(column = 0, row = 0, padx = 5, pady = 5)

    h = Hygrometer(frm)

    uc = ATMega_i(ser = open_ser_port())
    uc.add_callback(get_hyt, frame = TWOWIRE)

    hyt = HYT221()
    hyt.start()

    top.mainloop()