#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

# Humidity- und Temperatursensor HYT221

# hier: Auslesen durch Pollen der Seriellen Schnittstelle

# Es ist keine Adressierung von internen Registern erforderlich.
#
# Die Messung wird angestoßen durch die Adressierung des Sensors ohne Daten.
# Nach der Konversionzeit von 100ms können 4 Byte gelesen und interpretiert werden.
#
# Um den Abschluss der Konversion zu prüfen, werden im Byte[0] die oberen
# beiden Bits getestet, es darf keines der beiden Bits gesetzt sein.

import tkinter as Tk
import threading, time
from ser_port import open_ser_port
from global_h import *
from atmega_p import ATMega_p

class HYT221():
    """ Hygrosens HYT221 Sensor für Luftfeuchtigkeit und Temperatur """
    def __init__(self, twi_adr = 80, cb = None):
        self.twi_adr = twi_adr
        self.starttime = 0
        self.cb = cb


    def read_ready(self):
        """ prüft, ob die Wartezeit von 70ms abgelaufen ist.
            Wenn ja, dann kann der Messwert ausgelesen werden """
        if (time.time() - self.starttime > 0.1):
            return True
        else:
            return False


    def readdata(self):
        """ Liest Temperatur und Feuchtigkeitswert aus dem Sensor aus
            schreibt keine Adresse vor dem READ """
        while self.read_ready() == False:
            time.sleep(0.02)

        data = uc.twiread(self.twi_adr, cnt = 4)

        if data[0] & 0b11000000:
            print("error in data")

        data = ((data[0] * 256 + data[1], data[2]* 256 + data[3]))
        return self.calc(data)


    def calc(self, data):
        """ Berechnet die Temperatur aus dem gelieferten Messwert
            bei gesetztem höchstwertigen Bit liegen negative Temperaturen vor
            je Digit : 0.0625 Grad  """
        humi = round((data[0] * 100 / 16384), 1)
        temp = round((data[1]  * 165 / 65536 - 40 ),1)

        return (humi, temp)


    def startreading(self):
        """ Sendet nur die Adresse des Sensors
            Die Daten können nach 100ms abgeholt werden """
        self.starttime = time.time()
        uc.twiwrite(self.twi_adr)


class Hygrometer(object):

    def __init__(self, master):
        self.tvar = Tk.StringVar()
        self.hvar = Tk.StringVar()
        self.tvar.set("--")
        self.hvar.set("--")

        Tk.Label(master,text = "Humi:", width = 5, anchor = "e", font = ('Arial', 20)).grid(
                    column = 0, row = 0, padx = 5, pady = 5, sticky = "nse")
        Tk.Label(master,text = "Temp:", width = 5, anchor = "e", font = ('Arial', 20)).grid(
                    column = 0, row = 1, padx = 5, pady = 5, sticky = "nse")
        self.t = Tk.Label(master, width = 8, textvariable = self.hvar, font = ('Arial',20))
        self.t.grid(column = 1, row = 0, sticky = "nsw")
        self.t = Tk.Label(master, width = 8, textvariable = self.tvar, font = ('Arial',20))
        self.t.grid(column = 1, row = 1, sticky = "nsw")


    def set(self, value):
        self.hvar.set("{:3} %".format(value[0]))
        self.tvar.set("{:4.1f} C°".format(value[1]))


class Autosend(threading.Thread):
    """ Thread im Hintergrund, der in regelmäßigen Zeitabständen
        Aktionen ausführt
        hier: Ping an SRF02 senden, damit eine Entfernungsmessung ausgeführt wird
        """
    def __init__(self):
        threading.Thread.__init__(self)
        self.ende = False
        self.start()

    def run(self):
        while self.ende == False:
            hyt.startreading()
            data = hyt.readdata()
            h.set(data)
            time.sleep(.5)
        print("Thread Autosend stopped")
        exit()

    def stop(self):
        self.ende = True

if __name__ == "__main__":

    def shutdown(args = None):
        if ats.is_alive():
            ats.stop()

    top = Tk.Tk()
    top.title("HYT221")
    top.bind("<Destroy>", shutdown)
    top.resizable(width = False, height = False) # Größe nicht veränderbar

    frm = Tk.LabelFrame(top, text = "Temperatur- und Luftfeuchtigkeit")
    frm.grid(column = 0, row = 0, padx = 5, pady = 5)

    h = Hygrometer(frm)

    hyt = HYT221()

    uc = ATMega_p(ser = open_ser_port())

    ats = Autosend()

    top.mainloop()