#! /usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

import os, time
from ser_port import open_ser_port
from global_h import *
from atmega_p import ATMega_p


# Der PCF8583 kennt drei Betriebsarten:
# - clock -> Alarm, wenn Datum/Zeit erreicht
# - timer -> Alarm, wenn Alarm-Counter (Register 7) von 99 nach 0 überläuft
# - event counter -> zählt Impulse anstelle des Zeittaktes (Quarz/Netz)
#
# Die Register:
#
# 0  clock_control             8   alarm_control
# 1  Hundredth of second       9   Hundredth of second
# 2  Seconds                  10   Seconds
# 3  Minutes                  11   Minutes
# 4  Hours                    12   Hours
# 5  Year/Date                13   Year/Date
# 6  Weekday/Month            14   Weekday/Month
# 7  Timer                    15   Timer
#
# --- Register 0 - clock-control
# Bit 0       = Timer Flag
# Bit 1       = Alarm Flag
# Bit 2       = Alarm enable Bit
# Bit 3       = Mask Flag
# Bit 4 und 5 = Function Mode
# Bit 6       = Hold last count flag
# bit 7       = Stop counting flag

# --- Register 1 - 100tel Sekunden

# --- Register 2 - Sekunden
# Bit 7 bis 4 = Zehnerstellen der Sekunden [0 ... 9]
# Bit 3 bis 0 = Einerstellen der Sekunden  [0 ... 9]

# --- Register 3 - Minuten
# Bit 7 bis 4 = Zehnerstellen der Minuten  [0 ... 9]
# Bit 3 bis 0 = Einerstellen der Minuten   [0 ... 9]

# --- Register 4 - Stunden
# Bit 7 bis 6 = Flag für 24h Modus/Flag für AM/PM
# Bit 5 bis 4 = Zehnerstellen der Stunden  [0 ... 3]
# Bit 3 bis 0 = Einerstellen der Stunden   [0 ... 9]

# --- Register 5 - Datum
# Bit 7 bis 6 = Jahr                       [0 ... 3]
# Bit 5 bis 4 = Zehnerstellen des Datums   [0 ... 2]
# Bit 3 bis 0 = Einerstellen des Datums    [0 ... 9]

# --- Register 6 - Datum
# Bit 7 bis 5 = Weekday [ 0 ... 7]
# Bit       4 = Zehnerstellen des Monats   [0 ... 1]
# Bit 3 bis 0 = Einerstellen des Monats    [0 ... 9]

# --- Register 7 - Timer (Counter)

# --- Register 8 - alarm-control
# Bit 0 bis 2 = Timer function - Einheit
# Bit 3       = Timer Interrupt enable
# Bit 4 und 5 = Clock alarm function
# Bit 6       = Timer alarm enable
# Bit 7       = Alarm Interrupt enable

PCF8583_ADR        = 160
PCF_CLOCK_CTRL_REG = 0			       # Registeradressen des PCF8583
PCF_TIME_REG       = 1
PCF_DATE_REG       = 5
PCF_COUNTER_REG    = 7
PCF_ALARM_CTRL_REG = 8

# Der PCF8583 speichert die Jahreszahl nur in 2 Bit mit den Werten 0 ..
# Es wird als Ausgangspunkt das Jahr 2000 angenommen.
# Die zusätzlichen Jahre (z.B. 12 für 2012) werden im RAM des PCF8583 abgelegt.
# Hinzuaddiert wird die Jahreszahl [0..3] des PCF.
# Um einen Jahreswechsel zu erkennen, wird der Wert der PCF-Zeit zusätzlich
# im RAM abgelegt.
# Wird beim Auslesen der Zeit erkannt, dass die Jahreszahlt der PCF-Zeit und der
# Wert im RAM nicht übereinstimmen, dann wird ein Jahreswechsel stattgefunden haben,
# die Werte im RAM werden aktualisiert.
# für das Jahr 2013 sind im RAM angelegt:
# [12, 1]
# beim Überlauf auf das Jahr 2014 zeigt die Jahreszahl des PCF auf 2 (2000 + 12 + 2)
# es wird erkannt, dass der Wert im RAM vom der PCF-Zeit abweicht (größer ist),
# der Wert im RAM wird korrigiert [12, 1] -> [12, 2].
# Beim Jahreswechsel von 2016 auf 2017 wechselt das PCF-Jahr von 3 auf 0.
# Im RAM steht [12, 3], das PCF-Jahr [0] ist kleiner als die Jahreszahl im RAM [3].
# In diesem Fall wird die Werte im RAM korrigiert:
# [12, 3] neue Werte: [16, 0]

PCF_YEAR_OFFSET = 0x20	                 # Adressen von Daten im RAM des PCF8583

class PCF8583(object):
    def bcd_2_dez(self, bcd):
        """ Wandelt eine Zahl im BCD-FORMAT in das Dezimalformat um """
        i = (bcd >> 4) * 10             # Zehnerstelle freistellen und mit 10 multiplizieren
        i += bcd & 0x0F                 # Einerstelle freistellen und addieren
        return i

    def dez_2_bcd(self, dez):
        """ Wandelt eine Dezimalzahl [0 ... 99] in das BCD-Format um """
        i = (dez // 10) << 4            # Zehnerstelle freistellen und ins upper nibble
        i += dez % 10                   # Einerstelle ins lower nibble addieren
        return i

    def __init__(self, uc = None):

        self.__offset = 0               # (Jahr - 2000) - Jahr % 4
        self.__buffer = [0,0,0,0]       # Schreibpuffer
        self.__hour = 0
        self.__minute = 0
        self.__second = 0
        self.__year = 0
        self.__month = 0
        self.__day = 0
        self.uc = uc


    def settime(self, time):
        """ setzt die Uhrzeit - (Stunde, Minute, Sekunde) """
        self.__buffer[3] = self.dez_2_bcd(time[0])        # Stunde
        self.__buffer[2] = self.dez_2_bcd(time[1])        # Minute
        self.__buffer[1] = self.dez_2_bcd(time[2])        # Sekunde
        self.__buffer[0] = 0                              # hundertstel Sekunden

        #self.twi_send(WRITE, PCF8583_ADR, adr = PCF_TIME_REG, data = self.__buffer)
        self.uc.twiwrite(PCF8583_ADR, regadr = PCF_TIME_REG, data = self.__buffer)

    def setdate(self, date):
        """ setzt das Datum - ( Tag, Monat, Jahr)
            Jahr und Tag sind in einem einzigen Byte gespeichert.
            Das Jahr wird nur von 0 .. 3 verwaltet,
            0 ist ein Schaltjahr """
        # Der PCF8583 verwaltet nur Jahreszahlen von 0 .. 3
        # 0 ist ein Schaltjahr
        t = date[2] % 4
        self.__buffer[0] = t << 6
        # Der Offset zum Jahr 2000 + interner Jahreszahl (0..3)
        # wird im RAM des PCF8583 gespeichert.
        # Um einen Jahreswechsel erkennen zu können, wird auch
        # die interne Jahreszahl im RAM gespeichert
        # Ein Jahreswechsel wird daran erkannt, dass die interne
        # Jahreszahl nicht mehr mit der gespeicherten übereinstimmt.
        self.__offset = date[2] - t - 2000
#        self.twi_send(WRITE, PCF8583_ADR, adr = PCF_YEAR_OFFSET,
#                        data = [self.__offset, t])

        self.uc.twiwrite(PCF8583_ADR, regadr = PCF_YEAR_OFFSET, data = [self.__offset, t])
        self.__buffer[0] += (date[0] // 10) << 4
        self.__buffer[0] += (date[0] % 10)
        # der Wochentag wird nicht ausgewertet
        self.__buffer[1] = self.dez_2_bcd(date[1]) # Monat

        self.uc.twiwrite(PCF8583_ADR, regadr = PCF_DATE_REG, data = self.__buffer[0:2])
##        self.twi_send(WRITE, PCF8583_ADR, adr = PCF_DATE_REG,
##                         data = (self.__buffer[0:2]))


    def readtime(self):
        """ liest die Uhrzeit aus der RTC zurück """

       #time = self.twi_send(READ, PCF8583_ADR, cnt = 3, adr = PCF_TIME_REG + 1)
        time = self.uc.twiread(PCF8583_ADR, regadr = (PCF_TIME_REG + 1), cnt = 3)
        self.__second = self.bcd_2_dez(time[0])
        self.__minute = self.bcd_2_dez(time[1])
        self.__hour   = self.bcd_2_dez(time[2])


    def readdate(self):
        """ liest das Datum aus der RTC aus """

        #date = self.twi_send(READ, PCF8583_ADR, cnt = 2, adr = PCF_DATE_REG)
        date = self.uc.twiread(PCF8583_ADR, regadr = PCF_DATE_REG, cnt =  2)
        #self.__offset = self.twi_send(READ, PCF8583_ADR, cnt = 2, adr = PCF_YEAR_OFFSET)
        self.__offset = self.uc.twiread(PCF8583_ADR, regadr= PCF_YEAR_OFFSET, cnt = 2)
#        print(self.__offset)
        self.__offset = list(self.__offset)

        # t = die Jahresangabe des PCF8583 [0 ... 3]
        t = date[0] >> 6
        # Wenn diese Jahreszahl von dem gespeicherten Wert abweicht,
        # dann muss ein Jahreswechsel stattgefunden haben ...
        if not t == self.__offset[1]:
            # wenn t < als offset[1]
            # dann gibt es einen Überlauf des internen Zählers
            # dann muss der Offset korrigiert werden

            if t < self.__offset[1]:
                self.__offset[0] += 4
                self.__offset[1] = t

            # wenn t > als offset[1], dann wurde nur die interne im PCF8583
            # incrementiert, die ermittelte Jahreszahl bleibt korrekt.
            # Aber der neue Wert muss ins RAM des PCF8583 geschrieben werden.
            elif t > self.__offset[1]:
                self.__offset[1] = t

##            self.twi_send(WRITE, PCF8583_ADR, adr = PCF_YEAR_OFFSET,
##                        data = self.__offset)
            self.uc.twiwrite(PCF8583_ADR, regadr = PCF_YEAR_OFFSET, data = self.__offset)

        self.__year  = t + self.__offset[0] + 2000
        self.__month = self.bcd_2_dez(date[1] & 0x1F)
        self.__day   = self.bcd_2_dez(date[0] & 0x3F)


    def inputtime(self):
        """ Erlaubt die Zeit auf dem PCF8583 zu setzen """
        self.readdate()      # aktuelle Einstellung einlesen
        self.readtime()
                            # in lokale Variablen umkopieren
        date = [self.__day, self.__month, self.__year]
        time = [self.__second, self.__minute, self.__hour]

                            # Neue Werte einlesen
        x = input("Jahr [{}]: ".format(date[2]))
        if x.isdigit():
            date[2] = int(x)

        x = input("Monat [{}]: ".format(date[1]))
        if x.isdigit():
            date[1] = int(x)

        x  = input("Tag [{}]: ".format(date[0]))
        if x.isdigit():
            date[0] = int(x)


        x = input("Stunde [{}]: ".format(time[2]))
        if x.isdigit():
            time[2] = int(x)

        x = input("Minute [{}]: ".format(time[1]))
        if x.isdigit():
            time[1] = int(x)

        x  = input("Sekunde [{}]: ".format(time[0]))
        if x.isdigit():
            time[0] = int(x)

        print("{}-{}-{} {}:{}:{}".format(
             date[2], date[1], date[0], time[2], time[1], time[0]))

        x = input("Übernehmen [Y]")
        if len(x) == 0 or x.upper() == "Y":
            self.setdate((date[0], date[1], date[2]))
            self.settime((time[2], time[1], time[0]))
            print("PCF-Zeit übernommen")
            ret = True

        else:
            print("Abbruch")
            ret = False
        self.readdate()
        self.readtime()
        self.printtime()
        return ret

    def printtime(self):
        print("PCF8583-Time:  {:02}.{:02}.{:4} - {:02}:{:02}:{:02}".format(
                self.__day, self.__month, self.__year,
                self.__hour, self.__minute, self.__second))

    def gettime(self):
        return(self.__day, self.__month, self.__year,
                self.__hour, self.__minute, self.__second)


if __name__ == "__main__":

    mega = ATMega_p(open_ser_port(timeout = 0.1))

    t = PCF8583(uc = mega)

    #t.inputtime()      # hier kann die Zeit eingestellt werden
    t.readtime()
    t.readdate()

                        # zur Kontrolle die beiden Offsetwerte anzeigen
    offset = mega.twiread(PCF8583_ADR, cnt = 2, regadr = PCF_YEAR_OFFSET)

    print("Offset: {}, {}".format(offset[0], offset[1]))
    t.printtime()

    print("exit")