#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

# Funktionen zur Manipulation der Portregister,
# auf Rückmeldungen wird an der Seriellen Schnittstelle pollend gewartet.
# - Setzen von DDR, PORT und PIN
# - Auslesen der ADC's
# - Einlesen von Temperaturmesswerten mittels Sensor DS18B20
# - Write / Read auf dem TWI-Bus

import time
from ser_port import open_ser_port
from global_h import *

# --- Ports komplett schalten --------------------------------------------------

def port_mode(port, pinmask, pinstates):
    """ Einen oder mehrere Pins eines Ports als Ausgang oder Eingang setzen

        port (PORTB, PORTC, PORTD): 1, 2, 3
        pinmask   - Pins  des Ports, die geändert werden sollen
        pinstates - Status der Pins (im DDR-Register),
                    die in pinmask als zu ändernd angegeben sind

        pin_mode(0x01, 0x0F, 0x03) setzt an PORTB PB.0/PB1 als Ausgang und
                        PB.2/PB.3 als Eingang
        Rückgabe: keine """

    if not port in (1,2,3):
        raise SyntaxError("ungültiger Port")

    ser.write( (STARTBYTE, PORTMODE, 0x03, port, pinmask, pinstates) )


def port_read(port):
    """ Liest den Status der aller Pins des Ports

        port (PORTB, PORTC, PORTD): 1, 2, 3
        port_read(1) liest PortB

        Rückgabe: STARTBYTE, PORTREAD, 0x04, Port, DDR-Register, PORT-Register, PIN-Register
        """

    if not port in (1,2,3):
        raise SyntaxError("ungültiger Port")

    ser.write( (STARTBYTE, PORTREAD, 0x01, port) )
    return get_answer()


def port_write(port, pinmask, pinvalues):
    """ Setzt oder löscht einen oder mehrere Pins eines Ports

        port (PORTB, PORTC, PORTD): 1, 2, 3
        pinmask   - Pins  des Ports, die geändert werden sollen
        pinvalues - Werte der Pins, die in pinmask als zu ändernd angegeben sind

        pin_write(0x01, 0x0F, 0x3) setzt an PORTB PB.0/PB1 und löscht PB.2/PB.3

        Rückgabe: keine """

    if not port in (1,2,3):
        raise SyntaxError("ungültiger Port")

    ser.write( (STARTBYTE, PORTWRITE, 0x03, port, pinmask, pinvalues) )

def port_toggle(port, pinmask):
    """ Einen oder mehrere Pins eines Ports umschalten

        port (PORTB, PORTC, PORTD): 1, 2, 3
        pinmask   - Pins  des Ports, die geändert werden sollen

        pin_toggle(0x01, 0x04) wechselt den Status von PB.3

        Rückgabe: keine
        """
    if not port in (1,2,3):
        raise SyntaxError("ungültiger Port")
    ser.write( (STARTBYTE, PORTTOG, 0x02, port, pinmask) )

# --- Pins schalten durch Angabe von Port und Pin -----------------------------

def pin_mode(port, pin_nr, state):
    """ Schaltet den IO-Modus von genau einem Pin eines Ports um
        Paramter: Port, Pin_Nr, state [0 | <>0]
        Rückgabe: keine """
    check_port_pin(port, pin_nr)

    if state:
        state = pin_nr
    ser.write((STARTBYTE, PORTMODE, 0x03, port, pin_nr, state))


def pin_read(port, pin_nr):
    """ Liest den Status von genau einem Pin eines Ports
        Rückgabe Port, Pin, 0/1
        port (PORTB, PORTC, PORTD): 1, 2, 3
        pin_nr[0...7]

        Rückgabe: 0/1
        Fehlermeldung, wenn Port > 7 """

    check_port_pin(port, pin_nr)

    ser.write( (STARTBYTE, PORTREAD, 0x01, port) )
    a = get_answer()
    # Pin maskieren, liefert 0 oder 1 zurück
    a = (a[2] & (1<<pin_nr))
    if a > 0:
        a = 1
    return a

def pin_write(port, pin_nr, value):
    """ Schreibt genau einen Pin eines Ports
        Parameter: port, pin_nr, wert [0 | <>0]

        Rückgabe: keine
        Fehlermeldung, wenn Port > 7 """

    check_port_pin(port, pin_nr)

    pin_nr = (1<<pin_nr)
    if value:
        value = pin_nr

    ser.write((STARTBYTE, PORTWRITE, 0x03, port, pin_nr, value))

def pin_tog(port, pin_nr):
    """ Schaltet den Status um von genau einem Pin eines Ports
        Paramter: Port, Pin_Nr
        Rückgabe: keine """

    check_port_pin(port, pin_nr)

    ser.write( (STARTBYTE, PORTTOG, 0x02, port, (1<<pin_nr)) )


def check_port_pin(port, pin_nr):
    """ Pruft, ob die Angaben zu Port und Pin korrekt sind """
    if port is 1:
        if not (pin_nr >= 0 and pin_nr < 6):
            raise SyntaxError("PortB, ungültiger Pin: {}".format(pin_nr) )

    elif port is 2:
        if not (pin_nr >= 0 and pin_nr < 4):
            raise SyntaxError("PortC, ungültiger Pin: {}".format(pin_nr) )

    elif port is 3:
        if not (pin_nr >= 2 and pin_nr < 8):
            raise SyntaxError("PortD, ungültiger Pin: {}".format(pin_nr) )
    else:
        raise SyntaxError("unbekannter Port # {}".format(port))

# --- Pins durch Angabe einer PinNr. schalten ----------------------------------

# Für diejenigen, die wie beim Raspberry die Pins über eine Nr. ansprechen wollen,
# hier sind die entsprechenden Funktionen ....
# Verfügbar sind die Pins:
#  8,  9, 10, 11, 12, 13 (PortB, PB.0 bis PB.5
# 16, 17, 18, 19 (PortC, PC0. bis PC3)
# 26, 27, 28, 29, 30, 31 (PortD, PD2. bis PD.7)

def pinnr_mode(pin_nr, state):
    """ Wie pin_mode(), allerdings wird der Pin über seine Nummer angesprochen !
        Schaltet den IO-Modus von genau einem Pin eines Ports um
        Paramter: Port, Pin_Nr, state [0 | <>0]
        Rückgabe: keine
        Fehlermeldung, wenn ungültige Pinnr """

    check_pinnr(pin_nr)

    port = pin_nr // 8
    pin_nr = pin_nr % 8

    pin_nr = (1<<pin_nr)
    if state:
        state = pin_nr

    ser.write((STARTBYTE, PORTMODE, 0x03, port, pin_nr, state))


def pinnr_read(pin_nr):
    """ Wie pin_read, allerdings wird der PIn über seine Nummer angesprochen !
        Liest den Status von genau einem Pin eines Ports
        Rückgabe Port, Pin, True/False
        port (PORTB, PORTC, PORTD): 1, 2, 3
        pin_nr[0...7]

        Rückgabe: 0/1
        Fehlermeldung, wenn ungültige Pinnr """

    check_pinnr(pin_nr)

    port = pin_nr // 8
    pin_nr = pin_nr % 8

    ser.write( (STARTBYTE, PORTREAD, 0x01, port) )
    a = get_answer()
    # Pin maskieren, liefert 0 oder 1 zurück
    a = (a[2] & (1<<pin_nr))
    if a > 0:
        a = 1
    return a

def pinnr_write(pin_nr, value):
    """ Wie pin_write, allerdings wird der PIn über seine Nummer angesprochen !
        Schreibt genau einen Pin eines Ports
        Parameter: port, pin_nr, wert [0 | <>0]

        Rückgabe: keine
        Fehlermeldung, wenn ungültige Pinnr """

    check_pinnr(pin_nr)

    port = pin_nr // 8
    pin_nr = pin_nr % 8

    pin_nr = (1<<pin_nr)
    if value:
        value = pin_nr

    ser.write((STARTBYTE, PORTWRITE, 0x03, port, pin_nr, value))


def pinnr_tog(pin_nr):
    """ Wie pin_tog, allerdings wird der Pin über seine Nummer angesprochen !
        Schaltet den Status um von genau einem Pin eines Ports
        Paramter: Port, Pin_Nr
        Rückgabe: keine """

    check_pinnr(pin_nr)

    port = pin_nr // 8
    pin_nr = pin_nr % 8

    ser.write( (STARTBYTE, PORTTOG, 0x02, port, (1<<pin_nr)) )

def check_pinnr(pin_nr):
    """ Prüft, ob die angegebene Pinnummer existiert """
    if not pin_nr in (8, 9 , 10, 11, 12, 13, 16, 17, 18, 19, 26, 27, 28, 29, 30, 31):
        raise SyntaxError("unbekannter Pin: {}".format(pin_nr))

# --- Temperatursensor DS18B20 auslesen ----------------------------------------

def DS18B20_read(pin_mask):
    """ Liest DS18B20 Sensoren aus, die an PORTD angeschlossen sind.
        pinmask gibt an, an welchen Pins Sensoren gesucht werden sollen.
        Achtung: PD.0 und PD.1 sind wegen RX /TX nicht verfügbar

        readtemperatur(0x0C) liest die Sensoren an PD.2/PD.3 aus

        Rückgabe: STARTBYTE, ONEWIRE, 0x0?, pin_mask, t-high, t-low, ...)
        für jeden ausgelesenen Sensor zwei Datenbytes als 16-Bit Integer,
        der Temparaturwert ist jeweils durch 10 zu dividieren. """

    if pinmask & 0x03:
        raise SyntaxError("ungültige DS18B20-Pinmaske {:08b}".format(pin_mask))
    ser.write( (STARTBYTE, ONEWIRE, 0x01, pin_mask) )

# --- ADC auslesen -------------------------------------------------------------

def adc_read(pin_nr):
    """ Liest jeweils EINEN der vier verfügbaren ADC-Pin an PORTC aus.
        pinnr gibt an, an welchen Pins Sensoren gesucht werden sollen.
        Achtung: PC.4/PC.5 sind wegen SCL/SDA, PC.6 wegen Reset nicht verfügbar,
        PC.7 ist nicht vorhanden.
        ES kann immmer nur an einem einzigen Pin gemessen werden !
        read_ADC(0x00) liest den ADC an PC.0 aus

        Rückgabe: STARTBYTE, ADCREAD, 0x03, adc-high, adc-low """
    if pin_nr > 3:
        raise SyntaxError("ungültige ADC-PinNr")
    ser.write( (STARTBYTE, ADCREAD, 0x01, pin_nr) )

# --- Read/Write auf TWI-Bus ---------------------------------------------------

def twi_write(twiadr, data):
    """ Sendet an die TWI-Adresse "adr" das Iterable "data"
        adr = geradzahlige TWI-Adresse [0 .. 254]
              Das Bit.0 der Adresse wird immer gelöscht !
        data = Tuple, Liste """
    twiadr &= 0xFE
    if type(data) is int:
        data = (data,)

    out = [STARTBYTE, TWOWIRE, len(data) + 1, twiadr]
    for i in data:
        out.append(i)
    ser.write((out))


def twi_read(twiadr, cnt = 1, regadr = None):
    """ Sendet an die TWI-Adresse "adr" die Anforderung zum Lesen von Daten
        twiadr = TWI-Adresse des Slave
                 Das R/W-Bit.0 der Adresse wird automatisch gesetzt !
        cnt    = Anzahl der vom Slave zu lesenden Bytes <= 253 !
        regadr = optionale Registeradresse, die vorher an den Slave
                 gesendet wird (max. 2 Byte) !"""
    twiadr |= 0x01
    if cnt > 253:
        cnt = 253

    out = [STARTBYTE, TWOWIRE, 0, twiadr]
    if regadr is not None:
        if type(regadr) is int:
            regadr = (regadr,)
        if len(regadr) < 3:
            for i in regadr:
                out.append(i)

        else:
            raise SyntaxError("Registeradresse länger als 2 Byte: {}".format(regadr))
        out[2] = len(regadr) + 2
        out.append(cnt)

    else:
        out.append(cnt)
        out[2] = 2
    #print(out)
    ser.write( out )


def i2cdetect():
    """ Prüft, welche TWI-Devices am Bus antworten
        und gibt eine Liste mit den TWI-Adressen aus """
    print("i2cdetect:")
    for i in range(0, 256, 2):
        if (i % 16) == 0:
            print()
            print("0x{:02x}: ".format(i), end = "")
        ser.write((STARTBYTE, TWOWIRE, 0x01, i))
        ser.flushInput()
        if rs_read():
            print("0x{:02x} ".format(i), end = "")
        else:
            print("---- ", end = "")
    print()


# --- Rückmeldungen pollend abholen --------------------------------------------

def get_answer():
     """ Pollt die Serielle Schnittstelle für max. eine Sekunde.
         Die eingehenden Daten werden als Bytestring zurückgeben.
         Gehen innerhalb des Zeitfensters keine interpretierbaren Daten ein,
         wird ein IOError geworfen. """

     ser.flushInput()
     cnt = 200
     while cnt > 0:
         i = ser.inWaiting()
         # Mindestens 3 Byte müssen gelesen werden
         # um den Header zu idenifizieren und die nachfolgende
         # Datenlänge zu ermitteln.
         # Zuerst nur nach dem Anfang == IDENITFIER suchen
         if i > 0:
             v = ser.read(1)
             #print(v)
             # das erste Byte muss 0xA5 sein
             # wenn nicht, dann liegt kein gültiger Datensatz von
             if not v[0] == STARTBYTE:
                 continue
             # warten bis der Header komplett ist
             while ser.inWaiting() < 2:
                 time.sleep(0.001)

             # den restliche Header einlesen
             v = ser.read(2)
             # v[0] ist der Frametyp, der beim Pollen bekannt sein sollte
             # v[1] gibt die Anzahl der Datenbytes an

             d = ser.read(v[1])             # die Daten nach d einlesen

             if v[0] == TWI_ERR:
                print("TWI-Fehler @ adr: {:02x} !".format(d[0]))
                return (0,)

             elif v[0] == ADC_ERR:
                print("ungültiger ADC-Pin: {:08b}".format(d[0]))
                return (0,)

             elif v[0] == OWI_ERR:
                 print("Ungültige OWI-Maske: 0b{:02b}".format(d[0]))
                 return (0,)

             elif v[0] == ADC_ERR:
                 print("Ungültiger ADC-Pin: {}".format(d[0]))
                 return None

             elif v[0] == INT_ERR:
                 print("Ungültiger INT-Pin: {}".format(d[0]))
                 return None

             elif v[0] == TWOWIRE:      # eine TWI-Übertragung
                return d[1:]            # die TWI-Adresse abschneiden

             else:
                 return d

         time.sleep(0.005)
         cnt -= 1

     if cnt == 0:
         raise IOError("keine Antwort empfangen")


def print_portstate(port):
     """ Gibt den Status der Port-Register (DDRx, PORTx, PINx) aus """
     a = port_read(port)
     if len(a) is 4:
         print("Port{0}: DDR{0}: {1:08b}, PORT{0}: {2:08b}, PIN{0}: {3:08b}".
                 format(chr(a[0]+65), a[1], a[2], a[3]))
     else:
         print(len(a))

if __name__ == "__main__":

    ser = open_ser_port(timeout = 0.1)
    ser.flushInput()

    #i2cdetect()

##    for i in range(256):
##        twi_read(0xa2,  15, regadr = (0,i))
##        a = get_answer()
##        for i in a:
##            print(int(i), end = ", ")
##        print()

#PORTB testen
    port_mode(PORTB, 0xFF, 0x00)
    port_write(PORTB,0xFF, 0x0F)
#    port_mode(PORTB, 0xFF, 0xFF)

    print_portstate(PORTB)
    port_write(PORTB,0xFF, 0x00)
    print_portstate(PORTB)
#    time.sleep(.5)
##    port_write(PORTB,0xFF, 0x00)
##    time.sleep(.5)
##    port_write(PORTB,0xFF, 0xFF)
    port_mode(PORTB, 0xFF, 0xFF)
 #   time.sleep(.5)
    for j in range(2):
        #for i in range(8):
        for i in range(8, 14, 1):
            #pin_tog(PORTB, i)
            pinnr_write(i, j)
#            print_portstate(PORTB)
            time.sleep(.3)

# PORTC testen

##    port_write(PORTC,0x0F, 0xFF)
##    port_mode(PORTC, 0x0F, 0xFF)
##
##    for j in range(2):
##        #for i in range(4):
##        for i in range(16, 20, 1):
##            #pin_tog(PORTC, i)
##            pinnr_tog(i)
##            time.sleep(.5)

# PORTD testen

##    port_write(PORTD,0xFC, 0xFC)
##    port_mode(PORTD, 0xFC, 0xFC)
##
##    for j in range(2):
##        #for i in range(2, 8, 1):
##        for i in range(26, 32, 1):
##            #pin_tog(PORTD, i)
##            pinnr_tog(i)
##            time.sleep(.5)

    ser.close









