#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 12.08.2013

# SRF02 Ultrasonic range finder
# via TWI

# hier: Die Messwerte werden via Interrupt/Thread empfangen

# Achtung bei den TWI-Adressen:
# das TWI-Modul erwartet die Adressen linksbündig !

import tkinter as Tk
import threading, time
import adr_wahl

from ser_port import open_ser_port
from bargraph import BarGraph
from global_h import *
from atmega_i import ATMega_i

SRF_TWI_ADR	= 224 #0xE0 // 2 # 0x70
SRF_CMD_REG     = 0x00
#SRF_RANGE_REG	= 0x02

#SRF_RANGE_MODE_INCH = 0x50
SRF_RANGE_MODE_CM   = 0x51
#SRF_RANGE_MODE_USEC = 0x52

class SRF02(object):
    def __init__(self, twi_adr = SRF_TWI_ADR, ser = None):
        self.twi_adr = twi_adr
        self.starttime = 0
        self.ser = ser
        self.i = 1

    def send_ping(self):
        """ sendet das Command zur Messung in der Einheit cm
            merkt sich die Systemzeit, damit nach 70ms
            der Messwert ausgelesen werden kann."""
        self.starttime = time.time()
        uc.twiwrite( self.twi_adr, (SRF_CMD_REG,SRF_RANGE_MODE_CM) )
        self.busy = True

    def read_ready(self):
        """ prüft, ob die Wartezeit von 70ms abgelaufen ist.
            Wenn ja, dann kann der Messwert ausgelesen werden """
        if (time.time() - self.starttime > 0.07):
            return True
        else:
            return False

    def read_dist(self):
        """ liest die gemessene Distanz (nach Ablauf von 70ms) aus """
        # es wird einfach ab Register 1 gelesen,
        # aber nur die Werte von Register 2 und 3 ausgewertet
        uc.twiread(self.twi_adr, cnt = 3)


    def set_adr(self, adr):
        self.twi_adr = adr


class Autosend(threading.Thread):
    """ Thread im Hintergrund, der in regelmäßigen Zeitabständen
        Aktionen ausführt
        hier: Ping an den SRF02 senden, damit eine Entfernungsmessung ausgeführt wird
              nach einer Wartezeit von > 64ms das Auslesen der Daten anstoßen.
        """
    def __init__(self):
        threading.Thread.__init__(self)
        self.ende = False

    def run(self):
        while self.ende == False:
            srf.send_ping()

            # die Messdauer von 65ms abwarten
            while not srf.read_ready():
                time.sleep(0.05)
            # die Ausgabe des Messwert veranlassen
            # der Messwert wird im Hintergrund empfangen
            srf.read_dist()

            # die Zeit zwischen den Messungen
            # zuzüglich 75ms
            time.sleep(0.9)

        print("Thread Autosend stopped")
        exit()

    def stop(self):
        self.ende = True


if __name__ == "__main__":

    def bargraph_update(data):
        """ Callback zur Aktualisierung der Entfernungsanzeige.
            Die zurückgelieferten Daten sind:
            (TWI_Adresse, Konstante 0x06, Dist_high, Dist_low) """
        if data[0] == SRF_TWI_ADR + 1:
            bg.draw( data[2]*256 + data[3])

    def change_adr(adr):
        """ Callback zum Ändern der TWI-Adresse """
        srf.set_adr(adr)

    def shutdown(args = None):
        """ Beendet die beiden Hintergrundthreads """
        if uc.is_alive():        # Receiver beenden
            uc.stop()
        if a.is_alive():            # Autosend beenden
            a.stop()

    top = Tk.Tk()
    top.title("Ultraschall-Entfernungsmessung mit SRF02")
    top.bind("<Destroy>", shutdown)
    top.resizable(width = False, height = False) # Größe nicht veränderbar

    frm = Tk.LabelFrame(top, text = "Distanz in [cm]")
    frm.grid(column = 0, row = 0, padx = 5, pady = 5)

    # Bargraphen hinzufügen
    bg  = BarGraph(frm, start_value = 0, end_value = 500, width = 500,
                        heigth = 30, orientation = "e")
    bg.grid(column = 0, row = 0, padx = 5, pady = 5)

    # Adressauswahl hinzufügen
    aw = adr_wahl.Adr_wahl(top, startadr = 224, endadr = 254, adr = 224, cb = change_adr)
    aw.grid(column = 1, row = 0, padx = 5, pady = 5, sticky = "nsew")

    # Entfernungssensor
    srf = SRF02(twi_adr = SRF_TWI_ADR)

    # Hintergrundthread für den Empfang via Serieller Schnittstelle
    # dabei die serielle Schnittstelle öffnen
    uc = ATMega_i(ser = open_ser_port())

    uc.add_callback(bargraph_update, twiadr = SRF_TWI_ADR ) # an eine konkrete TWI-Adresse binden
    #uc.add_callback(bargraph_update, frame = TWOWIRE )  # allgemein an TWI-Antworten binden

    # Löst in regelmäßigen Zeitabständen Entfernungsmessungen aus
    a = Autosend()
    a.start()
    #uc.del_callback(frame = TWOWIRE)
    #uc.del_callback(twiadr = SRF_TWI_ADR)
    top.mainloop()